@@ -4,7 +4,7 @@
 #     A Python interface to Last.fm and Libre.fm
 #
 # Copyright 2008-2010 Amr Hassan
-# Copyright 2013-2017 hugovk
+# Copyright 2013-2018 hugovk
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,25 +31,24 @@ import tempfile
 import time
 import xml.dom
 
-__version__ = '2.0.0'
+__version__ = '2.2.0'
 __author__ = 'Amr Hassan, hugovk, Mice Pápai'
-__copyright__ = ('Copyright (C) 2008-2010 Amr Hassan, 2013-2017 hugovk, '
+__copyright__ = ('Copyright (C) 2008-2010 Amr Hassan, 2013-2018 hugovk, '
                  '2017 Mice Pápai')
 __license__ = "apache2"
 __email__ = 'amr.hassan@gmail.com'
 
 
-if sys.version_info[0] == 3:
-    import html.entities as htmlentitydefs
-    from http.client import HTTPSConnection
-    from urllib.parse import quote_plus as url_quote_plus
-
-    unichr = chr
-
-elif sys.version_info[0] == 2:
+if sys.version_info.major == 2:
     import htmlentitydefs
     from httplib import HTTPSConnection
     from urllib import quote_plus as url_quote_plus
+else:
+    import html.entities as htmlentitydefs
+    from http.client import HTTPSConnection
+    from urllib.parse import quote_plus as url_quote_plus
+    unichr = chr
+
 
 STATUS_INVALID_SERVICE = 2
 STATUS_INVALID_METHOD = 3
@@ -86,11 +85,12 @@ DOMAIN_RUSSIAN = 9
 DOMAIN_JAPANESE = 10
 DOMAIN_CHINESE = 11
 
-COVER_SMALL = 0
-COVER_MEDIUM = 1
-COVER_LARGE = 2
-COVER_EXTRA_LARGE = 3
-COVER_MEGA = 4
+# COVER_X is deprecated since 2.1.0 and will be removed in a future version
+SIZE_SMALL = COVER_SMALL = 0
+SIZE_MEDIUM = COVER_MEDIUM = 1
+SIZE_LARGE = COVER_LARGE = 2
+SIZE_EXTRA_LARGE = COVER_EXTRA_LARGE = 3
+SIZE_MEGA = COVER_MEGA = 4
 
 IMAGES_ORDER_POPULARITY = "popularity"
 IMAGES_ORDER_DATE = "dateadded"
@@ -712,6 +712,10 @@ class _ShelfCacheBackend(object):
     """Used as a backend for caching cacheable requests."""
     def __init__(self, file_path=None):
         self.shelf = shelve.open(file_path)
+        self.cache_keys = set(self.shelf.keys())
+
+    def __contains__(self, key):
+        return key in self.cache_keys
 
     def __iter__(self):
         return iter(self.shelf.keys())
@@ -720,13 +724,17 @@ class _ShelfCacheBackend(object):
         return self.shelf[key]
 
     def set_xml(self, key, xml_string):
+        self.cache_keys.add(key)
         self.shelf[key] = xml_string
 
 
 class _Request(object):
     """Representing an abstract web service operation."""
 
-    def __init__(self, network, method_name, params={}):
+    def __init__(self, network, method_name, params=None):
+
+        if params is None:
+            params = {}
 
         self.network = network
         self.params = {}
@@ -1045,11 +1053,14 @@ class _BaseObject(object):
             self, method, thing, thing_type, params=None, cacheable=True):
         """Returns a list of the most played thing_types by this thing."""
 
-        doc = self._request(
-            self.ws_prefix + "." + method, cacheable, params)
-
+        limit = params.get("limit", 1)
         seq = []
-        for node in doc.getElementsByTagName(thing):
+        for node in _collect_nodes(
+                limit,
+                self,
+                self.ws_prefix + "." + method,
+                cacheable,
+                params):
             title = _extract(node, "name")
             artist = _extract(node, "name", 1)
             playcount = _number(_extract(node, "playcount"))
@@ -1158,8 +1169,15 @@ class _Chartable(object):
 
         seq = []
         for node in doc.getElementsByTagName(chart_kind.lower()):
-            item = chart_type(
-                _extract(node, "artist"), _extract(node, "name"), self.network)
+            if chart_kind == "artist":
+                item = chart_type(
+                    _extract(node, "name"),
+                    self.network)
+            else:
+                item = chart_type(
+                    _extract(node, "artist"),
+                    _extract(node, "name"),
+                    self.network)
             weight = _number(_extract(node, "playcount"))
             seq.append(TopItem(item, weight))
 
@@ -1389,7 +1407,7 @@ class _Opus(_BaseObject, _Taggable):
         return (a == b) and (c == d)
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        return not self == other
 
     def _get_params(self):
         return {
@@ -1468,14 +1486,14 @@ class Album(_Opus):
     def __init__(self, artist, title, network, username=None):
         super(Album, self).__init__(artist, title, network, "album", username)
 
-    def get_cover_image(self, size=COVER_EXTRA_LARGE):
+    def get_cover_image(self, size=SIZE_EXTRA_LARGE):
         """
         Returns a uri to the cover image
         size can be one of:
-            COVER_EXTRA_LARGE
-            COVER_LARGE
-            COVER_MEDIUM
-            COVER_SMALL
+            SIZE_EXTRA_LARGE
+            SIZE_LARGE
+            SIZE_MEDIUM
+            SIZE_SMALL
         """
 
         return _extract_all(
@@ -1553,7 +1571,7 @@ class Artist(_BaseObject, _Taggable):
             return False
 
     def __ne__(self, other):
-        return not self.__eq__(other)
+        return not self == other
 
     def _get_params(self):
         return {self.ws_prefix: self.get_name()}
@@ -1575,15 +1593,15 @@ class Artist(_BaseObject, _Taggable):
         return _extract(
             self._request(self.ws_prefix + ".getCorrection"), "name")
 
-    def get_cover_image(self, size=COVER_MEGA):
+    def get_cover_image(self, size=SIZE_EXTRA_LARGE):
         """
         Returns a uri to the cover image
         size can be one of:
-            COVER_MEGA
-            COVER_EXTRA_LARGE
-            COVER_LARGE
-            COVER_MEDIUM
-            COVER_SMALL
+            SIZE_MEGA
+            SIZE_EXTRA_LARGE
+            SIZE_LARGE
+            SIZE_MEDIUM
+            SIZE_SMALL
         """
 
         return _extract_all(
@@ -1741,7 +1759,7 @@ class Country(_BaseObject):
         return self.get_name().lower() == other.get_name().lower()
 
     def __ne__(self, other):
-        return self.get_name() != other.get_name()
+        return not self == other
 
     def _get_params(self):  # TODO can move to _BaseObject
         return {'country': self.get_name()}
@@ -1869,7 +1887,7 @@ class Tag(_BaseObject, _Chartable):
         return self.get_name().lower() == other.get_name().lower()
 
     def __ne__(self, other):
-        return self.get_name().lower() != other.get_name().lower()
+        return not self == other
 
     def _get_params(self):
         return {self.ws_prefix: self.get_name()}
@@ -2008,13 +2026,17 @@ class Track(_Opus):
 
         self._request(self.ws_prefix + '.unlove')
 
-    def get_similar(self):
+    def get_similar(self, limit=None):
         """
         Returns similar tracks for this track on the network,
         based on listening data.
         """
 
-        doc = self._request(self.ws_prefix + '.getSimilar', True)
+        params = self._get_params()
+        if limit:
+            params['limit'] = limit
+
+        doc = self._request(self.ws_prefix + '.getSimilar', True, params)
 
         seq = []
         for node in doc.getElementsByTagName(self.ws_prefix):
@@ -2072,17 +2094,14 @@ class User(_BaseObject, _Chartable):
     def __str__(self):
         return self.get_name()
 
-    def __eq__(self, another):
-        if isinstance(another, User):
-            return self.get_name() == another.get_name()
+    def __eq__(self, other):
+        if isinstance(other, User):
+            return self.get_name() == other.get_name()
         else:
             return False
 
-    def __ne__(self, another):
-        if isinstance(another, User):
-            return self.get_name() != another.get_name()
-        else:
-            return True
+    def __ne__(self, other):
+        return not self == other
 
     def _get_params(self):
         return {self.ws_prefix: self.get_name()}
@@ -2287,8 +2306,8 @@ class User(_BaseObject, _Chartable):
 
         doc = self._request(self.ws_prefix + ".getInfo", True)
 
-        return doc.getElementsByTagName(
-            "registered")[0].getAttribute("unixtime")
+        return int(doc.getElementsByTagName(
+            "registered")[0].getAttribute("unixtime"))
 
     def get_tagged_albums(self, tag, limit=None, cacheable=True):
         """Returns the albums tagged by a user."""
@@ -2409,12 +2428,19 @@ class User(_BaseObject, _Chartable):
         return self._get_things(
             "getTopTracks", "track", Track, params, cacheable)
 
-    def get_image(self):
-        """Returns the user's avatar."""
+    def get_image(self, size=SIZE_EXTRA_LARGE):
+        """
+        Returns the user's avatar
+        size can be one of:
+            SIZE_EXTRA_LARGE
+            SIZE_LARGE
+            SIZE_MEDIUM
+            SIZE_SMALL
+        """
 
         doc = self._request(self.ws_prefix + ".getInfo", True)
 
-        return _extract(doc, "image")
+        return _extract_all(doc, "image")[size]
 
     def get_url(self, domain_name=DOMAIN_ENGLISH):
         """Returns the url of the user page on the network.
@@ -2595,7 +2621,7 @@ def _string(string):
     if isinstance(string, str):
         return string
     casted = six.text_type(string)
-    if sys.version_info[0] == 2:
+    if sys.version_info.major == 2:
         casted = casted.encode("utf-8")
     return casted
 
